// Copyright 2022-2023 The sacloud/iaas-api-go Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package main

import (
	"log"
	"path/filepath"

	"github.com/sacloud/iaas-api-go/internal/define"
	"github.com/sacloud/iaas-api-go/internal/tools"
)

const destination = "zz_apis.go"

func init() {
	log.SetFlags(0)
	log.SetPrefix("gen-api-interfaces: ")
}

func main() {
	tools.WriteFileWithTemplate(&tools.TemplateConfig{
		OutputPath: filepath.Join(tools.ProjectRootPath(), destination),
		Template:   tmpl,
		Parameter:  define.APIs,
	})
	log.Printf("generated: %s\n", filepath.Join(tools.ProjectRootPath(), destination))
}

const tmpl = `// generated by 'github.com/sacloud/iaas-api-go/internal/tools/gen-api-interfaces'; DO NOT EDIT

package iaas

import (
	"context"
	"github.com/sacloud/iaas-api-go/types"
)

{{ range . }} {{ $typeName := .TypeName }} {{ $resource := . }}

/************************************************* 
* {{ $typeName }}API
*************************************************/

// {{ $typeName }}API is interface for operate {{ $typeName }} resource
type {{ $typeName }}API interface {
{{ range .Operations }}
	{{ .MethodName }}(ctx context.Context{{if not $resource.IsGlobal}}, zone string{{end}}{{ range .Arguments }}, {{ .ArgName }} {{ .TypeName }}{{ end }}) {{.ResultsStatement}} 
{{- end -}}
}
{{ end }}
`
